// === App.js (client-side) ===
import Button from './components/Button.js';
import Modal from './components/Modal.js';
import Table from './components/Table.js';
import AddSiteModal from './modals/AddSiteModal.js';
import EditSiteModal from './modals/EditSiteModal.js';
import StartStopModal from './modals/StartStopModal.js';
import UpgradeVIPModal from './modals/UpgradeVIPModal.js';

// === API base URL (pastikan endpoint-nya benar) ===
const API_URL = "https://clota.me/api";

export default function App() {
  const [urls, setUrls] = React.useState([]);
  const [modalType, setModalType] = React.useState(null);
  const [modalData, setModalData] = React.useState(null);
  const [isVIP, setIsVIP] = React.useState(() => localStorage.getItem('isVIP') === 'true');
  const [clientId, setClientId] = React.useState(() => localStorage.getItem('clientId') || null);
  const [isOnline, setIsOnline] = React.useState(navigator.onLine);

  // --- Load VIP status from localStorage on mount ---
  React.useEffect(() => {
    const storedVIP = localStorage.getItem('isVIP');
    if (storedVIP === 'true') setIsVIP(true);
  }, []);

  // --- Save VIP status and clientId to localStorage when they change ---
  React.useEffect(() => {
    localStorage.setItem('isVIP', isVIP ? 'true' : 'false');
  }, [isVIP]);

  React.useEffect(() => {
    if (clientId) localStorage.setItem('clientId', clientId);
  }, [clientId]);

  // --- Register client on mount if not registered ---
  React.useEffect(() => {
    window.addEventListener('online', () => setIsOnline(true));
    window.addEventListener('offline', () => setIsOnline(false));

    if (!clientId) {
      registerClient();
    }

    return () => {
      window.removeEventListener('online', () => setIsOnline(true));
      window.removeEventListener('offline', () => setIsOnline(false));
    };
  }, [clientId]);

  async function registerClient() {
    const mode = isVIP ? 'vip1' : 'free';
    try {
      const response = await fetch(`${API_URL}/clients`, {
        method: 'POST',
        headers: { 'Content-Type': 'application/json' },
        body: JSON.stringify({ name: 'Anonymous', mode })
      });
  
      if (!response.ok) {
        const errorData = await response.json();
        console.error('❌ Server error:', errorData);
        return;
      }
  
      const data = await response.json();
      console.log('✅ Registered client:', data);
      setClientId(data.id);
    } catch (error) {
      console.error('❌ Client registration failed:', error);
    }
  }
  
  // --- Modal control ---
  function openModal(type, data = null) {
    setModalType(type);
    setModalData(data);
  }
  
  function closeModal() {
    setModalType(null);
    setModalData(null);
  }
  
  // --- Site Actions ---
  function handleAddSite(newSiteConfig) {
    const newSiteData = {
      id: urls.length + 1,
      ...newSiteConfig,
      visits: 0,
      credits: 0,
      status: 'Active',
      mode: isVIP ? 'vip' : 'free',
      clientID: clientId
    };
  
    setUrls(prevUrls => {
      const updatedUrls = [...prevUrls, newSiteData];
      window.electronAPI.startExchange(updatedUrls)
        .then(() => console.log('Exchange started.'))
        .catch((error) => console.error('Failed to start exchange:', error));
      return updatedUrls;
    });
  
    closeModal();
  }
  
  function handleEditSite(updatedSite) {
    const updatedList = urls.map(site => (site.id === updatedSite.id ? { ...site, ...updatedSite } : site));
    setUrls(updatedList);
    closeModal();
  }
  
  function handleStartStopSite(siteId, newStatus) {
    const updatedList = urls.map(site => (site.id === siteId ? { ...site, status: newStatus } : site));
    setUrls(updatedList);
    closeModal();
  }
  
  function handleDeleteSite(siteId) {
    const updatedList = urls.filter(site => site.id !== siteId);
    setUrls(updatedList);
  }
  
  function handleRunTestTools() {
    window.electronAPI.runTestTools(urls)
      .then((result) => console.log('Run Test Tools Result:', result))
      .catch((error) => console.error('Error running test tools:', error));
  }
  
  // --- Handle VIP Upgrade ---   
 
  // const handleUpgradeVIP = async (code) => {
  //   try {
  //     const res = await fetch(`${API_URL}/vip-code/use`, {
  //       method: 'POST',
  //       headers: { 'Content-Type': 'application/json' },
  //       body: JSON.stringify({ clientId, code }) // ✅ FIXED
  //     });
  
  //     const data = await res.json();
  
  //     if (res.ok && data.success) {
  //       setIsVIP(true);
  //       closeModal();
  //       return { success: true };
  //     } else {
  //       return { success: false, message: data.message };
  //     }
  //   } catch (err) {
  //     return { success: false, message: 'Network error' };
  //   }
  // };
  const handleUpgradeVIP = async (code) => {
    try {
      const res = await fetch(`${API_URL}/vip-code/use`, {
        method: 'POST',
        headers: { 'Content-Type': 'application/json' },
        body: JSON.stringify({ client_id: clientId, code }) // ✅ Diperbaiki
      });
  
      const data = await res.json();
  
      if (res.ok && data.success) {
        setIsVIP(true);
        closeModal();
        return { success: true };
      } else {
        return { success: false, message: data.message };
      }
    } catch (err) {
      return { success: false, message: 'Network error' };
    }
  };
  
  // --- Render modal content based on modalType ---
  function renderModalContent() {
    switch (modalType) {
      case 'add':
        return React.createElement(AddSiteModal, { onSave: handleAddSite, onCancel: closeModal });
      case 'edit':
        return React.createElement(EditSiteModal, { siteData: modalData, onSave: handleEditSite, onCancel: closeModal });
      case 'startstop':
        return React.createElement(StartStopModal, { siteData: modalData, onAction: handleStartStopSite, onCancel: closeModal });
      case 'upgrade':
        return React.createElement(UpgradeVIPModal, { onUpgrade: handleUpgradeVIP, onCancel: closeModal });
      default:
        return null;
    }
  }
  
  // --- Main Render ---
  return React.createElement(
    'div',
    { style: wrapperStyle },
    React.createElement(
      'div',
      { style: pageStyle },
      React.createElement('div', { id: 'header' }),
      React.createElement('div', { style: statusBarStyle },
        React.createElement('div', {},
          React.createElement('span', {}, 'Running Status: '),
          React.createElement('span', { style: isOnline ? statusNormal : statusOffline }, isOnline ? 'Online' : 'Offline'),
          React.createElement('span', { style: { marginLeft: '20px' } }, `Running Mode: ${isVIP ? 'VIP Mode' : 'Free Mode'}`),
          React.createElement('button', { style: linkButton, onClick: () => openModal('upgrade') }, 'Switch Mode')
        ),
        React.createElement('div', {},
          React.createElement('button', { style: controlButton, onClick: handleRunTestTools }, 'Run Test Tools')
        )
      ),
      React.createElement('div', { style: tableStaticContainer },
        React.createElement(Table, {
          data: urls,
          onEdit: (site) => openModal('edit', site),
          onStartStop: (site) => openModal('startstop', site),
          onDelete: handleDeleteSite
        })
      ),
      React.createElement('div', { style: addSiteArea },
        React.createElement('button', {
          style: addSiteButton,
          onClick: () => {
            if (!isVIP && urls.length >= 3) {
              alert('Free user maximum 3 URLs. Upgrade to VIP!');
              return;
            }
            if (isVIP && urls.length >= 10) {
              alert('VIP user maximum 10 URLs.');
              return;
            }
            openModal('add');
          }
        },
          `➕ Add Site (${urls.length}/${isVIP ? 10 : 3})`
        )
      ),
      React.createElement('div', { style: helpLink }, 'Visit Member Center for more information...'),
      modalType && React.createElement(Modal, { onClose: closeModal }, renderModalContent())
    )
  );
}

// === Styles ===
const wrapperStyle = {
  display: 'flex',
  justifyContent: 'center',
  alignItems: 'center',
  height: '100vh',
  backgroundColor: '#f0f0f0',
  margin: 0,
  padding: 0,
  overflow: 'hidden'
};

const pageStyle = {
  width: '900px',
  height: '600px',
  backgroundColor: '#fff',
  fontFamily: 'sans-serif',
  fontSize: '12px',
  borderRadius: '0px',
  overflow: 'hidden',
  boxShadow: '0 0 10px rgba(0,0,0,0.1)'
};

const statusBarStyle = {
  height: '40px',
  display: 'flex',
  justifyContent: 'space-between',
  alignItems: 'center',
  padding: '0 10px',
  borderBottom: '1px solid #ccc',
  fontSize: '12px'
};

const statusNormal = { color: 'green', fontWeight: 'bold' };
const statusOffline = { color: 'red', fontWeight: 'bold' };

const linkButton = {
  background: 'none',
  border: 'none',
  color: 'blue',
  textDecoration: 'underline',
  cursor: 'pointer',
  fontSize: '12px',
  marginLeft: '5px'
};

const controlButton = {
  backgroundColor: '#007bff',
  color: 'white',
  border: 'none',
  padding: '4px 8px',
  borderRadius: '5px',
  cursor: 'pointer',
  fontSize: '12px',
  marginLeft: '5px'
};

const tableStaticContainer = { height: '350px', overflow: 'hidden', padding: '0 10px' };

const addSiteArea = { height: '50px', display: 'flex', justifyContent: 'center', alignItems: 'center' };

const addSiteButton = {
  backgroundColor: '#4CAF50',
  color: 'white',
  padding: '8px 20px',
  fontSize: '14px',
  border: 'none',
  borderRadius: '5px',
  cursor: 'pointer'
};

const helpLink = {
  marginTop: '5px',
  textAlign: 'center',
  color: 'blue',
  textDecoration: 'underline',
  cursor: 'pointer',
  fontSize: '11px'
};
