// ===== src/renderer/app/services/exchangeService.js =====
const puppeteer = require('puppeteer-extra');
const StealthPlugin = require('puppeteer-extra-plugin-stealth');
const axios = require('axios');
const os = require('os');
const { smartLoop } = require('../../../main/navigator');
const config = require('../../../config/config');
const userAgents = require('../utils/userAgents');

puppeteer.use(StealthPlugin());

let runningInstances = [];
let visitedURLs = new Set();
let visitCounter = 0;

function calculateCredits(visits, mode = 'free') {
  return mode.startsWith('vip') ? visits : Math.floor(visits / 5);
}

async function getThreadCount() {
  const cores = os.cpus().length;
  return Math.max(1, Math.min(cores * 2, 10));
}

async function fetchTargetsFromServer(client_id) {
  try {
    const response = await axios.get(`${config.API_URL}/clients/targets`);
    return response.data.filter(target => target.client_id !== client_id);
  } catch (error) {
    console.error('❌ Gagal mengambil target dari server:', error.message);
    return [];
  }
}

async function submitSiteToServer(site) {
  try {
    await axios.post(`${config.API_URL}/clients/submit`, {
      client_id: site.client_id,
      url: site.url,
      keyword: site.keyword,
      searchEngine: site.searchEngine,
      targetDomain: site.targetDomain,
      delayMin: site.delayMin,
      delayMax: site.delayMax,
      sleepMin: site.sleepMin,
      sleepMax: site.sleepMax,
      scroll: site.scroll,
      clickNav: site.clickNav,
      clickSelector: site.clickSelector,
      loopCount: site.loopCount,
      navigationMode: site.navigationMode
    });
    console.log(`✅ ${site.url} submitted to server`);
  } catch (err) {
    console.error('❌ Submit gagal:', err.message);
  }
}

async function reportCreditsToServer(client_id, credits) {
  try {
    await axios.post(`${config.API_URL}/clients/credits`, { client_id, credits });
    console.log(`📤 ${credits} credits reported for client_id ${client_id}`);
  } catch (err) {
    console.error('❌ Gagal melaporkan kredit:', err.message);
  }
}

function getNextSite(targets) {
  for (const site of targets) {
    if (!visitedURLs.has(site.url)) {
      visitedURLs.add(site.url);
      return site;
    }
  }
  return null;
}

async function startExchange(client, clientSite) {
  const targets = await fetchTargetsFromServer(client.client_id);
  if (!targets.length) {
    console.log('❗ Tidak ada target aktif tersedia.');
    return;
  }

  const threadCount = await getThreadCount();
  console.log(`🧵 Menjalankan ${threadCount} thread...`);

  for (let i = 0; i < threadCount; i++) {
    const site = getNextSite(targets);
    if (!site) break;

    const browser = await puppeteer.launch({
      headless: true,
      executablePath: 'C:\\Program Files\\Google\\Chrome\\Application\\chrome.exe',
      args: ['--no-sandbox'],
      defaultViewport: null,
    });

    const page = await browser.newPage();
    const randomUserAgent = userAgents[Math.floor(Math.random() * userAgents.length)];
    await page.setUserAgent(randomUserAgent);

    (async () => {
      try {
        console.log(`🌐 Visiting: ${site.url} (mode: ${site.navigationMode || 'direct'})`);
        await smartLoop(page, site);

        const earnedCredits = calculateCredits(site.loopCount || 1, client.mode);
        visitCounter += site.loopCount || 1;

        console.log(`✅ Visited ${site.url} | Credits: ${earnedCredits} | Total Visits: ${visitCounter}`);
        await reportCreditsToServer(client.client_id, earnedCredits);

        if (client.mode.startsWith('vip') || (client.mode === 'free' && visitCounter % 5 === 0)) {
          await submitSiteToServer(clientSite);
        }

      } catch (err) {
        console.error(`❌ Gagal navigasi ke ${site.url}:`, err.message);
      } finally {
        await browser.close();
      }
    })();

    runningInstances.push(browser);
  }
}

async function stopExchange() {
  for (const browser of runningInstances) {
    try {
      await browser.close();
    } catch (err) {
      console.error('❌ Gagal menutup browser:', err.message);
    }
  }
  runningInstances = [];
  visitedURLs.clear();
  visitCounter = 0;
}

module.exports = {
  startExchange,
  stopExchange
};
