// /src/renderer/utils/deviceCheck.js

const os = require('os');

// ✅ Check Windows Version
function isWindows10OrHigher() {
  const release = os.release();
  const majorVersion = parseInt(release.split('.')[0], 10);
  return majorVersion >= 10;
}

// ✅ Detect RDP Session
function isRDP() {
  return process.env.SESSIONNAME && process.env.SESSIONNAME.startsWith('RDP-');
}

// ✅ Detect Virtual Machine (VM/Emulator)
function isVirtualMachine() {
  const platform = os.platform();
  const arch = os.arch();
  const release = os.release().toLowerCase();
  
  return release.includes('virtual') || arch.includes('ia32') || platform.includes('vm');
}

// ✅ Detect VPN or Proxy (based on network interfaces)
function hasProxyOrVPN() {
  const interfaces = os.networkInterfaces();
  for (const name of Object.keys(interfaces)) {
    for (const iface of interfaces[name]) {
      if (!iface.internal && iface.family === 'IPv4') {
        if (!(iface.address.startsWith('192.') || iface.address.startsWith('10.') || iface.address.startsWith('172.'))) {
          return true;
        }
      }
    }
  }
  return false;
}

// ✅ Validate Device (Combine all)
async function validateDevice() {
  if (process.platform !== 'win32' || !isWindows10OrHigher()) {
    console.error('Blocked: OS not Windows 10+');
    return false;
  }
  if (isRDP()) {
    console.error('Blocked: RDP session detected');
    return false;
  }
  if (isVirtualMachine()) {
    console.error('Blocked: Virtual Machine detected');
    return false;
  }
  if (hasProxyOrVPN()) {
    console.error('Blocked: Proxy/VPN detected');
    return false;
  }
  return true;
}

module.exports = {
  validateDevice
};
