// /src/renderer/utils/pageInteraction.js

// Helper sleep function
function sleep(ms) {
  return new Promise(resolve => setTimeout(resolve, ms));
}

// Smooth scroll to bottom
async function smoothScroll() {
  return new Promise((resolve) => {
    let totalHeight = 0;
    const distance = 100;
    const timer = setInterval(() => {
      window.scrollBy(0, distance);
      totalHeight += distance;

      if (totalHeight >= document.body.scrollHeight) {
        clearInterval(timer);
        resolve();
      }
    }, 100);
  });
}

// Click navigation element
async function clickNav(selector) {
  return new Promise((resolve) => {
    const element = document.querySelector(selector);
    if (element) {
      element.click();
      console.log('✅ Clicked Nav:', selector);
    } else {
      console.warn('⚠️ Nav Element not found:', selector);
    }
    resolve();
  });
}

// Click specific target element
async function clickSelector(selector) {
  return new Promise((resolve) => {
    const element = document.querySelector(selector);
    if (element) {
      element.click();
      console.log('✅ Clicked Selector:', selector);
    } else {
      console.warn('⚠️ Selector not found:', selector);
    }
    resolve();
  });
}

// Set modern Windows 2023+ UserAgent
function setModernUserAgent() {
  const newUserAgent = "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:117.0) Gecko/20100101 Firefox/117.0";
  Object.defineProperty(navigator, 'userAgent', {
    get: () => newUserAgent
  });
}

// Main interact function
export async function interactWithPage(config) {
  const {
    delay = 0,
    sleepTime = 0,
    scrollEnabled = false,
    clickNavEnabled = false,
    navSelector = '',
    clickSelectorEnabled = false,
    selector = '',
    loopCount = 1
  } = config;

  console.log('🌐 Starting page interaction with config:', config);

  // 1. Set modern UserAgent
  setModernUserAgent();
  console.log('🌐 UserAgent overridden to Windows 10/11 (2023+).');

  // 2. Loop Actions
  for (let i = 0; i < loopCount; i++) {
    console.log(`🔁 Loop ${i + 1}/${loopCount}`);

    // (a) Delay
    if (delay > 0) {
      console.log(`⏳ Delay ${delay}s before starting actions`);
      await sleep(delay * 1000);
    }

    // (b) Scroll
    if (scrollEnabled) {
      console.log('📜 Scrolling to bottom...');
      await smoothScroll();
    }

    // (c) Click Navigation
    if (clickNavEnabled && navSelector) {
      console.log(`🖱️ Clicking navigation selector: ${navSelector}`);
      await clickNav(navSelector);
    }

    // (d) Click Specific Selector
    if (clickSelectorEnabled && selector) {
      console.log(`🖱️ Clicking target selector: ${selector}`);
      await clickSelector(selector);
    }

    // (e) Sleep after actions
    if (sleepTime > 0) {
      console.log(`😴 Sleeping ${sleepTime}s between loops`);
      await sleep(sleepTime * 1000);
    }
  }

  console.log('✅ Finished all interaction loops!');
}
